from wii_roms_tool.src import download_vimms_rom, download_romsfun_rom, search_for_rom_vimms, search_for_rom_romsfun


def main():
    print("Welcome to Wii-Rom-Downloader!")
    print("1: Vimms Lair")
    print("2: Romsfun (Recommended)")

    while True:
        answer = input("Which do you choose? ")

        if answer == "1":
            while True:
                url = search_for_rom_vimms()
                if url is None:
                    pass
                else:
                    download_vimms_rom(url)
                    break
            break
        elif answer == "2":
            while True:
                url = search_for_rom_romsfun()
                if url is None:
                    pass
                else:
                    download_romsfun_rom(url)
                    break
            break
        else:
            print("Invalid input. Please enter 1 or 2.")


if __name__ == "__main__":
    main()



import os
import sys
import requests
import urllib3

from tqdm import tqdm
from bs4 import BeautifulSoup
from playwright.sync_api import sync_playwright
from difflib import SequenceMatcher

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

def get_vimms_id(url):
    response = requests.get(url, timeout=15, verify=False)

    soup = BeautifulSoup(response.text, 'html.parser')
    serial_id = soup.find(id="serials").text
    middle_part = serial_id.split('-')[1]
    result = middle_part + "01"
    return result


def download_file(url: str, installer_path: str, headers = None):
    try:
        if headers:
            response = requests.get(url, stream=True, headers=headers)
        else:
            response = requests.get(url, stream=True)
        total_size = int(response.headers.get('content-length', 0))
        block_size = 1024
        t = tqdm(total=total_size, unit='B', unit_scale=True)
        with open(installer_path, 'wb') as f:
            for data in response.iter_content(block_size):
                t.update(len(data))
                f.write(data)
        t.close()
        print(f"Download was successful!")
    except Exception as e:
        print(f"Error while downloading: {e}")
        if os.path.exists(installer_path):
            os.remove(installer_path)
        sys.exit(1)


def get_gametdb_id(query):
    def choose_region():
        print("Choose the right Region for the rom:")
        print("1. Europa (Australia)")
        print("2. USA")
        print("3. Japan")

        while True:
            region_choice = input("Choose (1/2/3): ")

            if region_choice == "1":
                return "P"
            elif region_choice == "2":
                return "E"
            elif region_choice == "3":
                return "J"
            else:
                print("Invalid Choice!")

    def similarity(a, b):
        return SequenceMatcher(None, a.lower(), b.lower()).ratio()

    url = f"https://www.gametdb.com/Main/Results?q={query}"

    with sync_playwright() as p:
        browser = p.chromium.launch(headless=True)
        page = browser.new_page()

        page.goto(url)

        page.wait_for_selector(".gs-title", timeout=20000)

        title_containers = page.locator(".gs-title")

        best_match = None
        best_similarity = 0

        for i in range(title_containers.count()):
            title_element = title_containers.nth(i)
            html = title_element.inner_html()

            soup = BeautifulSoup(html, "html.parser")
            a_tag = soup.find("a")

            if not a_tag:
                continue

            link = a_tag.get("href")
            clean_title = a_tag.get_text().strip()

            score = similarity(clean_title, query)
            if score > best_similarity:
                best_similarity = score
                best_match = (clean_title, link)

        browser.close()

    if best_match:
        print(f"Best title match: {best_match[0]} with a similarity of {best_similarity:.2f}!")
    else:
        print("No matching title!")
        return None

    if best_match[1]:
        game_id = best_match[1].strip("/").split("/")[-1]
        trimmed_id = game_id[:-3]
        game_id = trimmed_id + choose_region() + "01"
        return game_id
    return None


if __name__ == "__main__":
    print(get_gametdb_id("Mario Kart Wii"))



import curses
import urllib3
import requests
from bs4 import BeautifulSoup

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def fetch_links(query):
    url = f"https://vimm.net/vault/?p=list&system=Wii&q={query}"
    response = requests.get(url, timeout=15, verify=False)
    soup = BeautifulSoup(response.content, "html.parser")

    links = []
    for a in soup.find_all('a', href=True):
        if a.parent.name == 'td' and 'width:auto' in a.parent.get('style', ''):
            title = a.text.strip()
            link = f"https://vimm.net{a['href']}"

            row = a.find_parent('tr')
            if row:
                regions = [img['title'] for img in row.find_all('img', class_='flag') if 'title' in img.attrs]
                region_text = ", ".join(regions) if regions else "Unknown"
                extras_box = row.find('b', class_='redBorder')
                extras_text = f" ({extras_box['title']})" if extras_box and 'title' in extras_box.attrs else ""
            else:
                region_text = "Unknown"
                extras_text = ""
            if "manual" not in link:
                links.append((f"{title} ({region_text}){extras_text}", link))

    if not links:
        raise ValueError("No results found.")

    return links


def curses_menu(stdscr, links):
    curses.curs_set(0)
    curses.start_color()
    curses.init_pair(1, curses.COLOR_BLACK, curses.COLOR_WHITE)
    current_row = 0

    while True:
        stdscr.clear()
        stdscr.addstr(0, 0, "Top 20 Results:", curses.A_BOLD)

        for idx, (title, _) in enumerate(links[:20]):
            stdscr.addstr(idx + 2, 0, title, curses.color_pair(1)
                          if idx == current_row else 0)

        stdscr.refresh()
        key = stdscr.getch()

        if key == curses.KEY_UP and current_row > 0:
            current_row -= 1
        elif key == curses.KEY_DOWN and current_row < len(links[:20]) - 1:
            current_row += 1
        elif key in (10, 13):
            return links[current_row][1]
        elif key == 27:
            return None


def main(query):
    links = fetch_links(query)
    return curses.wrapper(curses_menu, links)


def search_for_rom_vimms():
    while True:
        answer = input("What rom do you want to download? ")
        if len(answer.replace(" ", "")) >= 3:
            return main(answer)
        else:
            print("You need to type in 3 or more letters!")


if __name__ == "__main__":
    search_for_rom_vimms()



import os
import platform
import subprocess
import sys
import tarfile
import zipfile
import cloudscraper
import py7zr
import urllib3

from bs4 import BeautifulSoup
from playwright.sync_api import sync_playwright
from wii_roms_tool.src import get_vimms_id, download_file, get_gametdb_id

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def download_romsfun_rom(url):
    print("Preparing...")
    download_path = os.path.join(os.path.expanduser("~"), "Downloads")
    rom_downloader_path = os.path.join(download_path, "Rom-Downloader")
    os.makedirs(rom_downloader_path, exist_ok=True)

    scraper = cloudscraper.create_scraper()
    response = scraper.get(url)

    soup = BeautifulSoup(response.text, 'html.parser')
    button = soup.select_one('.btn.btn-primary.btn-block.mx-auto.mb-4.small')

    if button:
        url = button.get('href')
    else:
        raise ValueError("Button-Element not found!")

    response = scraper.get(url)
    soup = BeautifulSoup(response.text, 'html.parser')
    rom_links = soup.select('div.bg-white.border.rounded.shadow-sm.py-2.px-3.mb-4 table tbody tr td a')

    print("Choose the ROM file you want to download:")
    for index, rom in enumerate(rom_links, start=1):
        print(f"{index}. {rom.text.strip()}")

    selected_link = None
    try:
        choice = int(input("Enter the number of the ROM you want to download: "))
        if 1 <= choice <= len(rom_links):
            selected_link = rom_links[choice - 1]['href']
        else:
            print("Invalid choice. Please try again.")
    except ValueError:
        print("Invalid input. Please enter a number.")

    print("Starting to fetch...")
    with sync_playwright() as p:
        browser = p.chromium.launch(headless=True)
        page = browser.new_page()
        page.goto("https://example.com")
        original_user_agent = page.evaluate("() => navigator.userAgent")
        clean_user_agent = original_user_agent.replace("HeadlessChrome", "Chrome")
        context = browser.new_context(
            user_agent=clean_user_agent
        )
        page = context.new_page()

        page.goto(selected_link, wait_until='domcontentloaded')
        page.wait_for_selector('a#download')
        download_link = page.query_selector('a#download').get_attribute('href')
        browser.close()

    game_id = 'game_id'
    zip_path = os.path.join(rom_downloader_path, game_id + ".zip")

    download_file(download_link, zip_path)

    extract_rename_folders(game_id, zip_path)


def download_vimms_rom(url):
    print("Preparing...")
    with sync_playwright() as p:
        download_path = os.path.join(os.path.expanduser("~"), "Downloads")
        rom_downloader_path = os.path.join(download_path, "Rom-Downloader")
        os.makedirs(rom_downloader_path, exist_ok=True)

        browser = p.chromium.launch(headless=True)
        context = browser.new_context(accept_downloads=True)
        page = context.new_page()
        page.goto(url)

        soup = BeautifulSoup(page.content(), "html.parser")

        select_version = soup.find("select", {"id": "dl_version"})
        select_format = soup.find("select", {"id": "dl_format"})

        if select_version:
            versions = [option.text for option in select_version.find_all("option")]
            print("Available versions:")
            for idx, version in enumerate(versions):
                print(f"{idx}: {version}")
            version_choice = int(input("Choose version by number: "))
            selected_version = select_version.find_all("option")[version_choice]["value"]
            if len(versions) != 1:
                page.select_option("#dl_version", selected_version)
            print(f"Selected version: {versions[version_choice]}")
        else:
            print("Version selector not found.")

        if select_format:
            formats = [option.text for option in select_format.find_all("option")]
            print("Available formats:")
            for idx, fmt in enumerate(formats):
                print(f"{idx}: {fmt}")
            format_choice = int(input("Choose format by number: "))
            selected_format = select_format.find_all("option")[format_choice]["value"]
            if len(formats) != 1:
                page.select_option("#dl_format", selected_format)
            print(f"Selected format: {formats[format_choice]}")
        else:
            print("Format selector not found.")
            print("This rom is currently not downloadable!")
            sys.exit()

        with page.expect_download() as download_info:
            page.evaluate("setMediaId('dl_form', allMedia)")
            page.evaluate(f"setFormat('dl_form', '{selected_format}', allMedia);")
            page.evaluate("confirmPopup(document.forms['dl_form'], 'tooltip4');")

        download = download_info.value
        download_url = download.url

        user_agent = page.evaluate("() => navigator.userAgent")

        cookies = context.cookies()
        cookie_header = "; ".join([f"{cookie['name']}={cookie['value']}" for cookie in cookies])

        headers = {
            "User-Agent": user_agent,
            "Referer": url,
            "Cookie": cookie_header,
        }

        game_id = get_vimms_id(url)
        zip_path = os.path.join(rom_downloader_path, game_id + ".7z")

        browser.close()

    download_file(download_url, zip_path, headers)

    extract_rename_folders(game_id, zip_path)


def download_and_extract_wit():
    def locate_wit(extract_dir):
        for root, dirs, files in os.walk(extract_dir):
            if dirs:
                first_folder = dirs[0]
                wit_dir = os.path.join(root, first_folder, "bin")
                for root2, _, files2 in os.walk(wit_dir):
                    for file in files2:
                        if file.lower() == "wit.exe" or file == "wit":
                            wit_executable = os.path.join(root2, file)
                            print(f"Found WIT at: {wit_executable}")
                            return wit_executable
        return None

    system = platform.system()
    base_url = "https://wit.wiimm.de/download"

    if system == "Windows":
        filename = "wit-v3.05a-r8638-cygwin64.zip"
    elif system == "Linux":
        filename = "wit-v3.05a-r8638-x86_64.tar.gz"
    elif system == "Darwin":
        filename = "wit-v3.05a-r8638-mac.tar.gz"
    else:
        raise OSError(f"Unsupported OS: {system}")

    download_url = f"{base_url}/{filename}"
    appdata_path = os.path.join(
        os.getenv("APPDATA") or os.path.expanduser("~"), "wii-roms-tool"
    )
    archive_path = os.path.join(appdata_path, filename)
    extract_dir = os.path.join(appdata_path, "wit_tool")
    os.makedirs(extract_dir, exist_ok=True)
    wit_executable = locate_wit(extract_dir)

    if wit_executable:
        return wit_executable

    print(f"Downloading WIT from...")
    download_file(download_url, archive_path)

    print("Extracting archive ...")
    if filename.endswith(".zip"):
        with zipfile.ZipFile(archive_path, 'r') as zip_ref:
            zip_ref.extractall(extract_dir)
    elif filename.endswith(".tar.gz"):
        with tarfile.open(archive_path, 'r:gz') as tar_ref:
            tar_ref.extractall(extract_dir)
    else:
        raise ValueError("Unsupported archive format")
    os.remove(archive_path)
    wit_executable = locate_wit(extract_dir)
    if wit_executable:
        return wit_executable

    raise FileNotFoundError("WIT executable not found in the archive.")


def download_and_extract_dolphin():
    def locate_dolphin(extract_dir):
        for root, dirs, files in os.walk(extract_dir):
            if dirs:
                first_folder = dirs[0]
                dolphin_dir = os.path.join(root, first_folder)
                for root2, _, files2 in os.walk(dolphin_dir):
                    for file in files2:
                        if file == "DolphinTool.exe":
                            dolphin_executable = os.path.join(root2, file)
                            print(f"Found dolphin at: {dolphin_executable}")
                            return dolphin_executable
        return None

    base_url = "https://dl.dolphin-emu.org/releases/2503/dolphin-2503-x64.7z"
    filename = "dolphin-2503-x64.7z"
    appdata_path = os.path.join(
        os.getenv("APPDATA") or os.path.expanduser("~"), "wii-roms-tool"
    )
    archive_path = os.path.join(appdata_path, filename)
    extract_dir = os.path.join(appdata_path, "dolphin")
    os.makedirs(extract_dir, exist_ok=True)
    dolphin_executable = locate_dolphin(extract_dir)

    if dolphin_executable:
        return dolphin_executable

    print(f"Downloading dolphin from...")
    download_file(base_url, archive_path)

    print("Extracting archive ...")
    with py7zr.SevenZipFile(archive_path, 'r') as zip_ref:
        zip_ref.extractall(extract_dir)
    os.remove(archive_path)

    dolphin_executable = locate_dolphin(extract_dir)
    if dolphin_executable:
        return dolphin_executable

    raise FileNotFoundError("dolphin executable not found in the archive.")


def extract_rename_folders(game_id: str, zip_path: str):
    print("Extracting Folders...")
    if ".7z" in zip_path:
        folder_path = zip_path.replace(".7z", "")
        with py7zr.SevenZipFile(zip_path, mode="r") as archive:
            archive.extractall(folder_path)
            print("Folders extracted!")
    else:
        folder_path = zip_path.replace(".zip", "")
        with zipfile.ZipFile(zip_path, mode="r") as archive:
            archive.extractall(folder_path)
            print("Folders extracted!")

    print("Removing zipfiles...")
    os.remove(zip_path)
    print("Removed zipfile!")

    print("Renaming in id...")
    game = "NamingError"
    for file in os.listdir(folder_path):
        file_path = os.path.join(folder_path, file)

        if os.path.isfile(file_path):
            name, ext = os.path.splitext(file)
            if not ".txt" in ext:
                game = name.split("(")[0]
            else:
                os.remove(file_path)
                continue
            if game_id == "game_id":
                game_id = get_gametdb_id(game)

            new_name = f"{game_id}{ext}"
            new_path = os.path.join(folder_path, new_name)

            os.rename(file_path, new_path)
            print(f"Renamed: {file} → {new_name}")
    if "game_id" in folder_path:
        os.rename(folder_path, folder_path.replace("game_id", f"{game}[{game_id}]"))
        end_file = os.path.join(folder_path.replace("game_id", f"{game}[{game_id}]"), new_name)
    else:
        os.rename(folder_path, folder_path.replace(game_id, f"{game}[{game_id}]"))
        end_file = os.path.join(folder_path.replace(game_id, f"{game}[{game_id}]"), new_name)
    print("All files have been renamed.")

    convert_to_wbfs(end_file)


def convert_to_wbfs(file_path):
    while True:
        answer = input("Do you want to convert to wbfs? (Real wii hardware) (y/n): ").strip().lower()
        if answer in ("yes", "y"):
            break
        elif answer in ("no", "n"):
            return
        else:
            print("Please answer 'yes' or 'no'.")
    if file_path.endswith(".wbfs"):
        print("Provided file is already a wbfs file!")
        return
    if platform.system() != "Windows":
        print("Currently no support on your os!")
        return
    wit_path = download_and_extract_wit()
    dolphin_path = download_and_extract_dolphin()
    iso_path = file_path[:-4] + ".iso"

    print("Converting to iso!...")
    try:
        subprocess.run([dolphin_path, "convert", "-i", file_path, "-o", iso_path, "-f", "iso"], check=True)
        os.remove(file_path)
        print(f"Conversion complete: {iso_path}")
    except subprocess.CalledProcessError as e:
        print("WIT error:", e)
    except Exception as e:
        print("Unexpected error:", e)

    print("Converting to wbfs!...")
    try:
        subprocess.run([wit_path, "convert", iso_path, "--wbfs"], check=True)
        print(f"Conversion complete!")
    except subprocess.CalledProcessError as e:
        print("WIT error:", e)
    except Exception as e:
        print("Unexpected error:", e)



import curses
import cloudscraper
import urllib3
from bs4 import BeautifulSoup

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def fetch_links(query):
    url = f"https://romsfun.com/roms/nintendo-wii/?s={query}"
    scraper = cloudscraper.create_scraper()
    response = scraper.get(url)
    soup = BeautifulSoup(response.content, "html.parser")

    links = []
    for div in soup.find_all('div', class_='col-archive-rom'):
        a = div.find('a', href=True)
        title_tag = div.find('h3')
        if a and title_tag:
            title = title_tag.text.strip()
            link = a['href']
            if not link.startswith("http"):
                link = "https://romsfun.com" + link
            links.append((title, link))

    if not links:
        raise ValueError("No results found.")

    return links


def curses_menu(stdscr, links):
    curses.curs_set(0)
    curses.start_color()
    curses.init_pair(1, curses.COLOR_BLACK, curses.COLOR_WHITE)
    current_row = 0

    while True:
        stdscr.clear()
        stdscr.addstr(0, 0, "Top 16 Results:", curses.A_BOLD)

        for idx, (title, _) in enumerate(links[:20]):
            stdscr.addstr(idx + 2, 0, title, curses.color_pair(1) if idx == current_row else 0)

        stdscr.refresh()
        key = stdscr.getch()

        if key == curses.KEY_UP and current_row > 0:
            current_row -= 1
        elif key == curses.KEY_DOWN and current_row < len(links[:16]) - 1:
            current_row += 1
        elif key in (10, 13):
            return links[current_row][1]
        elif key == 27:
            return None


def main(query):
    links = fetch_links(query)
    return curses.wrapper(curses_menu, links)


def search_for_rom_romsfun():
    while True:
        answer = input("What rom do you want to download? ")
        if len(answer.replace(" ", "")) >= 3:
            return main(answer)
        else:
            print("You need to type in 3 or more letters!")


if __name__ == "__main__":
    pass



from .common import get_vimms_id, download_file, get_gametdb_id
from .downloads import download_vimms_rom, download_romsfun_rom
from .search_vimms import search_for_rom_vimms
from .search_romsfun import search_for_rom_romsfun
